/* Miscellaneous functions.
 * src/misc.c
 * Copyright (C) 2000-2001 DarkSiderz Net.  All rights reserved.
 * Read the file doc/contact.html for contact information related to 
 * this software. */

/*************************************************************************/

#ifndef MISC_C
#define MISC_C

/*************************************************************************/

#include "services.h"

/*************************************************************************/

static char months[12][4] = { 
   "Jan", "Feb", "Mar", "Apr", "May", "Jun",
   "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
};

/*************************************************************************/

/* toupper()
 * Like the ANSI functions, but make sure we return an int instead of 
 * a (signed) char. */

int toupper(char c)
{
   if (islower(c))
      return (unsigned char)c - ('a'-'A');
   else
      return (unsigned char)c;
}

/*************************************************************************/

/* tolower()
 * Like the ANSI functions, but make sure we return an int instead of 
 * a (signed) char. */

int tolower(char c)
{
   if (isupper(c))
      return (unsigned char)c + ('a'-'A');
   else
      return (unsigned char)c;
}

/*************************************************************************/

/* sgets2()
 * Read a line of text from a socket, and strip newline and
 * carriage return characters from the end of the line. */

char *sgets2(char *buf, long size, int sock)
{
   char *s = sgets(buf, size, sock);

   if (!s || s == (char *)-1)
      return s;
   s = buf + strlen(buf)-1;
   if (*s == '\n')
      *s-- = 0;
   if (*s == '\r')
      *s = 0;
   return buf;
}

/*************************************************************************/

/* strscpy()
 * Copy at most len-1 characters from a string to a buffer, and
 * add a null terminator after the last character copied. */

char *strscpy(char *d, const char *s, size_t len)
{
   char *d_orig = d;

   if (!len)
      return d;
   while (--len && (*d++ = *s++))
   ;
   *d = 0;
   return d_orig;
}

/*************************************************************************/

/* stristr()
 * Search case-insensitively for string s2 within string s1,
 * returning the first occurrence of s2 or NULL if s2 was not
 * found. */

char *stristr(char *s1, char *s2)
{
   register char *s = s1, *d = s2;

   while (*s1) {
      if (tolower(*s1) == tolower(*d)) {
         s1++;
	   d++;
         if (*d == 0)
            return s;
      } else {
	   s = ++s1;
	   d = s2;
	}
   }
   return NULL;
}

/*************************************************************************/

/* merge_args()
 * Take an argument count and argument vector and merge them into a 
 * single string in which each argument is separated by a space. */

char *merge_args(int argc, char **argv)
{
   int i;
   static char s[4096];
   char *t;

   t = s;
   for (i = 0; i < argc; i++)
      t += snprintf(t, sizeof(s) - (t - s), "%s%s", *argv++, (i < argc - 1) ? " " : "");
   return s;
}

/*************************************************************************/

/* match_wild()
 * Attempt to match a string to a pattern which might contain '*' or 
 * '?' wildcards.  Return 1 if the string matches the pattern, 0 if not. */

static int do_match_wild(const char *pattern, const char *str, int docase)
{
   char c;
   const char *s;

   /* This WILL eventually terminate: either by *pattern == 0, or by a
    * trailing '*'. */

   for (;;) {
      switch (c = *pattern++) {
	   case 0:
	      if (!*str)
		   return 1;
            return 0;
	   case '?':
	      if (!*str)
		   return 0;
            str++;
	      break;
         case '*':
            /* A trailing '*' matches everything else in the sequence. */
	      if (!*pattern)
		   return 1;	
	      s = str;
	      while (*s) {
		   if ((docase ? (*s == *pattern) : (tolower(*s) == tolower(*pattern))) && do_match_wild(pattern, s, docase))
		    return 1;
               s++;
	      }
	      break;
	   default:
	      if (docase ? (*str++ != c) : (tolower(*str++) != tolower(c)))
		   return 0;
	   break;
	} 
   }
}

/*************************************************************************/

/* match_wild()
 * Does the given text match the given string? */

int match_wild(const char *pattern, const char *str)
{
   return do_match_wild(pattern, str, 1);
}

/*************************************************************************/

/* match_wild_nocase()
 * Does the given text match the given string ignorning case? */

int match_wild_nocase(const char *pattern, const char *str)
{
   return do_match_wild(pattern, str, 0);
}

/*************************************************************************/

/* month_name()
 * Return the three-character month name corresponding to the
 * given month number, 1-12. */

char *month_name(int month)
{
   return months[month-1];
}

/*************************************************************************/

/* strupper()
 * Convert the given string to upper case. */

char *strupper(char *s)
{
   char *t = s;
   while (*t)
      *t++ = toupper(*t);
   return s;
}

/*************************************************************************/

/* strlower()
 * Convert the given string to lower case. */

char *strlower(char *s)
{
   char *t = s;
   while (*t)
      *t++ = tolower(*t);
   return s;
}

/*************************************************************************/

/* read_string()
 * Read a string from a file, with the string length prefixed as a 
 * two-byte big-endian integer.  The filename is passed in so that
 * it can be reported in the log file if an error occurs. */

char *read_string(FILE *f, const char *filename)
{
   char *s;
   int len;

   len = fgetc(f) * 256 + fgetc(f);
   s = smalloc(len);
   if (len != fread(s, 1, len, f))
      fatal_perror("Read error on %s", filename);
   return s;
}

/*************************************************************************/

/* write_string()
 * Write a string to a file, with the string length prefixed as a 
 * two-byte big-endian integer.  The filename is passed in so that
 * it can be reported in the log file if an error occurs. */

void write_string(const char *s, FILE *f, const char *filename)
{
   int len;

   /* Include trailing null */
   len = strlen(s) + 1;	
   fputc(len / 256, f); fputc(len & 255, f);
   if (len != fwrite(s, 1, len, f))
      fatal_perror("Write error on %s", filename);
}

/*************************************************************************/

#endif

/*************************************************************************/